@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * Callbacks for widgets events
@@ -208,43 +209,71 @@ on_button_info_return_clicked (GtkButton *but, gpointer user_data)
 /**********************************************************************
  * 3 - Tutor window
  **********************************************************************/
-#define CB_COLOR_TAG(TAG, FGCOLOR, BGCOLOR) \
-	if (main_preferences_exist ("colors", TAG "_fg"))\
-		color_fg = main_preferences_get_string ("colors", TAG "_fg");\
-	else\
-	{\
-		color_fg = g_strdup (FGCOLOR);\
-		main_preferences_set_string ("colors", TAG "_fg", color_fg);\
-	}\
-	if (main_preferences_exist ("colors", TAG "_bg"))\
-		color_bg = main_preferences_get_string ("colors", TAG "_bg");\
-	else\
-	{\
-		color_bg = g_strdup (BGCOLOR);\
-		main_preferences_set_string ("colors", TAG "_bg", color_bg);\
-	}\
-	gtk_text_buffer_create_tag (buf, TAG,\
-		       	"foreground", color_fg,\
-		       	"background", color_bg,\
-		       	"underline", PANGO_UNDERLINE_NONE, NULL);\
-	g_free (color_bg);\
+
+static void 
+cb_color_tag(gchar *tag, gchar *fgcolor, gchar *bgcolor, GtkTextBuffer *tbuf)
+{
+	gchar *tmp;
+	gchar *color_fg;
+	gchar *color_bg;
+
+	tmp = g_strconcat (tag, "_fg", NULL);
+	if (main_preferences_exist ("colors", tmp))
+		color_fg = main_preferences_get_string ("colors", tmp);
+	else
+	{
+		color_fg = g_strdup (fgcolor);
+		main_preferences_set_string ("colors", tmp, color_fg);
+	}
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+	{
+		g_free (color_fg);
+		color_fg = main_altcolor_get_string ("colors", tmp);
+	}
+	g_free (tmp);
+
+	tmp = g_strconcat (tag, "_bg", NULL);
+	if (main_preferences_exist ("colors", tmp))
+		color_bg = main_preferences_get_string ("colors", tmp);
+	else
+	{
+		color_bg = g_strdup (bgcolor);
+		main_preferences_set_string ("colors", tmp, color_bg);
+	}
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+	{
+		g_free (color_bg);
+		color_bg = main_altcolor_get_string ("colors", tmp);
+	}
+	g_free (tmp);
+
+	gtk_text_buffer_create_tag (tbuf, tag,
+		       	"foreground", color_fg,
+		       	"background", color_bg,
+		       	"underline", PANGO_UNDERLINE_NONE, NULL);
 	g_free (color_fg);
+	g_free (color_bg);
+}
 
 G_MODULE_EXPORT void
 on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 {
 	gboolean beep;
 	gchar *tmp;
-	gchar *search;
 	gchar *tmp_font;
-	gchar *color_fg;
-	gchar *color_bg;
+	gchar *search;
 	gchar *color_main_fg;
 	gchar *color_main_bg;
 	GtkWidget *wg;
 	GtkTextBuffer *buf;
 	PangoFontDescription *font_desc;
+	PangoStyle pg_style;
+	PangoWeight pg_weight;
+	gint pg_size;
+	gchar *pg_family;
 	GdkRGBA color;
+	GtkStyleContext *sc;
+	static GtkCssProvider *css;
 
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
 
@@ -258,9 +287,16 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 		color_main_fg = main_preferences_get_string ("colors", "char_untouched_fg");
 	else
 		color_main_fg = g_strdup (TUTOR_BLACK);
+	/* Check if altcolor applies */
+	if (main_altcolor_get_boolean ("colors", "altcolor") == TRUE)
+	{
+		g_free (color_main_fg);
+		g_free (color_main_bg);
+		color_main_fg = main_altcolor_get_string ("colors", "char_untouched_fg");
+		color_main_bg = main_altcolor_get_string ("colors", "char_untouched_bg");
+	}
 
-	/*
-	 * Colors of text on the tutor window (note: ordering here matters, the first tag created is in the bottom!)
+	/* Colors of text on the tutor window (note: ordering here matters, the first tag created is in the bottom!)
 	 */
 	gtk_text_buffer_create_tag (buf, "char_keep_wrap",
 		       	"background", color_main_bg,
@@ -272,13 +308,13 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 		       	"foreground", color_main_fg,
 		       	"underline", PANGO_UNDERLINE_NONE, NULL);
 
-	CB_COLOR_TAG ("char_untouched",	TUTOR_BLACK,	TUTOR_CREAM);
-	CB_COLOR_TAG ("char_wrong", 	TUTOR_RED,	TUTOR_RED_LITE);
-	CB_COLOR_TAG ("char_correct",	TUTOR_GREEN,	TUTOR_CREAM);
-	CB_COLOR_TAG ("char_retouched",	TUTOR_BROWN,	TUTOR_GRAY);
-	CB_COLOR_TAG ("cursor_blink",	TUTOR_BLACK,	TUTOR_YELLOW);
-	CB_COLOR_TAG ("cursor_unblink",	TUTOR_BLACK,	TUTOR_CREAM);
-	CB_COLOR_TAG ("text_intro",	TUTOR_BLACK,	TUTOR_WHITE);
+	cb_color_tag ("char_untouched",	TUTOR_BLACK,	TUTOR_CREAM,    buf);
+	cb_color_tag ("char_wrong", 	TUTOR_RED,	TUTOR_RED_LITE, buf);
+	cb_color_tag ("char_correct",	TUTOR_GREEN,	TUTOR_CREAM,    buf);
+	cb_color_tag ("char_retouched",	TUTOR_BROWN,	TUTOR_GRAY,     buf);
+	cb_color_tag ("cursor_blink",	TUTOR_BLACK,	TUTOR_YELLOW,   buf);
+	cb_color_tag ("cursor_unblink",	TUTOR_BLACK,	TUTOR_CREAM,    buf);
+	cb_color_tag ("text_intro",	TUTOR_BLACK,	TUTOR_WHITE,    buf);
 
 	/* Tutor font */
 	tmp_font = main_preferences_get_string ("tutor", "lesson_font");
@@ -288,35 +324,37 @@ on_text_tutor_realize (GtkWidget * widget, gpointer user_data)
 		main_preferences_set_string ("tutor", "lesson_font", tmp_font);
 	}
 	gtk_text_buffer_create_tag (buf, "lesson_font", "font", tmp_font, NULL);
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (get_wg ("fontbutton_tutor")), tmp_font);
+	gtk_font_chooser_set_font (GTK_FONT_CHOOSER (get_wg ("fontbutton_tutor")), tmp_font);
 
-	/* Change default font throughout the widget */
+	/* Change default font throughout the widget
+	 */
 	font_desc = pango_font_description_from_string (tmp_font);
 	g_free (tmp_font);
-	gtk_widget_override_font (widget, font_desc);
+	pg_style = pango_font_description_get_style (font_desc);
+	pg_weight = pango_font_description_get_weight (font_desc);
+	pg_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
+	pg_family = (gchar *) pango_font_description_get_family (font_desc);
+	tmp = g_strdup_printf (".tutor-font {color: %s; font: %s %s %ipt %s;}", color_main_fg, 
+			pg_style==PANGO_STYLE_NORMAL?"":"italic", 
+			pg_weight>PANGO_WEIGHT_MEDIUM?"bold":"",
+		       	pg_size, pg_family);
+	css = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (css, tmp, -1, NULL);
+	sc = gtk_widget_get_style_context (widget);
+	gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gtk_style_context_add_class (sc, "tutor-font");
 	pango_font_description_free (font_desc);
+	g_free (tmp);
 
-	/* Change default background color throughout the widget */
-	gdk_rgba_parse (&color, color_main_bg);
-	gtk_widget_override_background_color (widget, GTK_STATE_FLAG_INSENSITIVE, &color);
+	g_free (color_main_bg);
+	g_free (color_main_fg);
 
-	/* Change default text color throughout the widget */
-	gdk_rgba_parse (&color, color_main_fg);
-	gtk_widget_override_color (widget, GTK_STATE_FLAG_INSENSITIVE, &color);
-
-	/* Turns on/off the beeps according to last time
+	/* Turns on/off the beeps according to preferences
 	 */
 	if (main_preferences_exist ("tutor", "tutor_beep"))
 		beep = main_preferences_get_boolean ("tutor", "tutor_beep");
 	else
 		beep = TRUE;
-	wg = get_wg ("togglebutton_tutor_beep");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wg), beep);
-	main_preferences_set_boolean ("tutor", "tutor_beep", beep);
-
-
-	g_free (color_main_bg);
-	g_free (color_main_fg);
 
 	tutor_init_goals ();
 }
@@ -342,7 +380,12 @@ on_button_tutor_other_clicked (GtkButton * button, gpointer user_data)
 	wg = get_wg ("treeview_other");
 	list = GTK_LIST_STORE ( gtk_tree_view_get_model (GTK_TREE_VIEW (wg)) );
 	if (tutor_get_type () == TT_VELO)
-		tutor_load_list_other (".words", list);
+	{
+		if (main_velo_txt ())
+			tutor_load_list_other (".paragraphs", list);
+		else
+			tutor_load_list_other (".words", list);
+	}
 	else if (tutor_get_type () == TT_FLUID)
 		tutor_load_list_other (".paragraphs", list);
 
@@ -350,25 +393,6 @@ on_button_tutor_other_clicked (GtkButton * button, gpointer user_data)
 	other_renaming_undo = g_strdup ("*");
 }
 
-G_MODULE_EXPORT void
-on_button_tutor_stat_clicked (GtkButton * button, gpointer user_data)
-{
-	gint aux;
-
-	gtk_combo_box_set_active (GTK_COMBO_BOX (get_wg ("combobox_stat_module")), tutor_get_type ());
-	if (tutor_get_type () == TT_BASIC)
-	{
-		callbacks_shield_set (TRUE);
-		aux = basic_get_lesson () - (basic_get_lesson_increased () ? 1 : 0);
-		aux += (aux == 0) ? 1 : 0;
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (get_wg ("spinbutton_stat_lesson")), aux);
-		callbacks_shield_set (FALSE);
-	}
-
-	plot_draw_chart (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_type"))) + 1);
-	gtk_widget_show (get_wg ("window_stat"));
-}
-
 G_MODULE_EXPORT void
 on_spinbutton_lesson_value_changed (GtkSpinButton * spinbutton, gpointer user_data)
 {
@@ -392,6 +416,71 @@ on_spinbutton_lesson_value_changed (GtkSpinButton * spinbutton, gpointer user_da
 	tutor_process_touch (L'\0');
 }
 
+G_MODULE_EXPORT void
+on_eventbox_lesson_plus (GtkWidget * widget, gpointer user_data)
+{
+	gint tmp_int;
+	GtkSpinButton *spb;
+	
+	spb = GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson"));
+	tmp_int = gtk_spin_button_get_value_as_int (spb);
+	gtk_spin_button_set_value (spb, tmp_int + 1);
+}
+
+G_MODULE_EXPORT void
+on_eventbox_lesson_minus (GtkWidget * widget, gpointer user_data)
+{
+	gint tmp_int;
+	GtkSpinButton *spb;
+	
+	spb = GTK_SPIN_BUTTON (get_wg ("spinbutton_lesson"));
+	tmp_int = gtk_spin_button_get_value_as_int (spb);
+	if (tmp_int > 0)
+		gtk_spin_button_set_value (spb, tmp_int - 1);
+}
+
+G_MODULE_EXPORT void
+on_eventbox_stat_plus (GtkWidget * widget, gpointer user_data)
+{
+	gint tmp_int;
+	GtkSpinButton *spb;
+	
+	spb = GTK_SPIN_BUTTON (get_wg ("spinbutton_stat_lesson"));
+	tmp_int = gtk_spin_button_get_value_as_int (spb);
+	gtk_spin_button_set_value (spb, tmp_int + 1);
+}
+
+G_MODULE_EXPORT void
+on_eventbox_stat_minus (GtkWidget * widget, gpointer user_data)
+{
+	gint tmp_int;
+	GtkSpinButton *spb;
+	
+	spb = GTK_SPIN_BUTTON (get_wg ("spinbutton_stat_lesson"));
+	tmp_int = gtk_spin_button_get_value_as_int (spb);
+	if (tmp_int > 0)
+		gtk_spin_button_set_value (spb, tmp_int - 1);
+}
+
+G_MODULE_EXPORT void
+on_button_tutor_stat_clicked (GtkButton * button, gpointer user_data)
+{
+	gint aux;
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (get_wg ("combobox_stat_module")), tutor_get_type ());
+	if (tutor_get_type () == TT_BASIC)
+	{
+		callbacks_shield_set (TRUE);
+		aux = basic_get_lesson () - (basic_get_lesson_increased () ? 1 : 0);
+		aux += (aux == 0) ? 1 : 0;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (get_wg ("spinbutton_stat_lesson")), aux);
+		callbacks_shield_set (FALSE);
+	}
+
+	plot_draw_chart (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_type"))) + 1);
+	gtk_widget_show (get_wg ("window_stat"));
+}
+
 G_MODULE_EXPORT void
 on_togglebutton_edit_basic_lesson_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
@@ -488,11 +577,14 @@ on_fontbutton_tutor_font_set (GtkFontButton * fbut, gpointer user_data)
 	GtkTextIter end;
 	gchar *tmp_font;
 
-	tmp_font = g_strdup (gtk_font_button_get_font_name (fbut));
+	tmp_font = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (fbut));
 	if (tmp_font == NULL)
 		tmp_font = g_strdup (LESSON_FONT);
 	if (strlen (tmp_font) == 0)
+	{
+		g_free (tmp_font);
 		tmp_font = g_strdup (LESSON_FONT);
+	}
 	main_preferences_set_string ("tutor", "lesson_font", tmp_font);
 
 	wg = get_wg ("text_tutor");
@@ -508,12 +600,15 @@ on_fontbutton_tutor_font_set (GtkFontButton * fbut, gpointer user_data)
 }
 
 G_MODULE_EXPORT void
-on_togglebutton_tutor_beep_toggled (GtkToggleButton * togglebutton, gpointer user_data)
+on_togglebutton_altcolor_toggled (GtkToggleButton * togglebutton, gpointer user_data)
 {
-	gboolean beep;
+	gboolean acol;
 
-	beep = gtk_toggle_button_get_active (togglebutton);
-	main_preferences_set_boolean ("tutor", "tutor_beep", beep);
+	if (callbacks_shield)
+		return;
+
+	acol = gtk_toggle_button_get_active (togglebutton);
+	main_altcolor_set_boolean ("colors", "altcolor", acol);
 }
 
 G_MODULE_EXPORT void
@@ -565,12 +660,6 @@ on_button_tutor_close_clicked (GtkButton *button, gpointer user_data)
 	on_window_tutor_destroy (NULL, NULL);
 }
 
-G_MODULE_EXPORT void
-on_button_tutor_restart_clicked (GtkButton * button, gpointer user_data)
-{
-	cb_quick_restart ();
-}
-
 G_MODULE_EXPORT void
 on_eventbox_tutor_restart_grab_focus (GtkWidget * widget, gpointer user_data)
 {
@@ -728,6 +817,23 @@ on_entry_mesg_drag_drop (GtkWidget * widget, GdkDragContext * drag_context,
 	return FALSE;
 }
 
+G_MODULE_EXPORT void
+on_entry_mesg_realize (GtkWidget * widget, gpointer user_data)
+{
+	GtkBindingSet *bs;
+	GdkModifierType modmask = GDK_CONTROL_MASK;
+
+	bs = gtk_binding_set_by_class (G_OBJECT_GET_CLASS ( G_OBJECT (GTK_ENTRY (widget))));
+	gtk_binding_entry_add_signal (bs, GDK_KEY_w, modmask, 
+			"delete-from-cursor", 2, 
+			G_TYPE_INT, GTK_DELETE_WORDS,
+			G_TYPE_INT, -1); /* Delete -1 word: Ctrl+w */
+	gtk_binding_entry_add_signal (bs, GDK_KEY_h, modmask, 
+			"delete-from-cursor", 2, 
+			G_TYPE_INT, GTK_DELETE_CHARS,
+			G_TYPE_INT, -1); /* Delete -1 char: Ctrl+h */
+}
+
 /**********************************************************************
  * 4 - Top 10 management
  **********************************************************************/
@@ -765,7 +871,7 @@ on_combobox_top10_language_changed (GtkComboBox *cmb, gpointer user_data)
 	if (tutor_get_type () == TT_FLUID)
 		tutor_init (tutor_get_type ());
 
-	on_combobox_top10_changed (GTK_COMBO_BOX (get_wg ("combobox_top10_language")), NULL);
+	on_combobox_top10_changed (GTK_COMBO_BOX (get_wg ("combobox_top10")), NULL);
 }
 
 G_MODULE_EXPORT void
@@ -774,7 +880,7 @@ on_button_top10_go_www_clicked         (GtkButton       *button,
 {
 	gchar *url;
 	url = gtk_widget_get_tooltip_text (get_wg ("button_top10_go_www"));
-	gtk_show_uri (NULL, url, GDK_CURRENT_TIME, NULL);
+	gtk_show_uri_on_window (get_win ("window_top10") , url, GDK_CURRENT_TIME, NULL);
 	g_free (url);
 }
 
@@ -888,7 +994,7 @@ on_button_kb_remove_clicked (GtkButton * button, gpointer user_data)
 	tmp_path = g_strconcat (main_path_user (), G_DIR_SEPARATOR_S, keyb_get_name (), ".kbd", NULL);
 	if (! g_file_test (tmp_path, G_FILE_TEST_IS_REGULAR))
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default());
 		g_message ("no valid keyboard to remove...");
 		g_free (tmp_path);
 		return;
@@ -1067,6 +1173,8 @@ on_dialog_confirm_show (GtkWidget * widget, gpointer user_data)
 {
 	gchar *action;
 	gchar *msg;
+	gchar *tmp;
+	gint len;
 	GtkLabel *wg_label;
 
 	wg_label = GTK_LABEL (get_wg ("label_confirm_action"));
@@ -1092,10 +1200,28 @@ on_dialog_confirm_show (GtkWidget * widget, gpointer user_data)
 		gtk_label_set_text (wg_label, msg);
 		g_free (msg);
 	}
+	else if (g_str_equal (action, "RESET_ALL"))
+	{
+		gtk_window_set_title (GTK_WINDOW (widget), _("Reset progress data"));
+		gtk_label_set_text (wg_label, _("This will DELETE all the progress data of all modules."));
+	}
 	else if (g_str_equal (action, "RESET"))
 	{
 		gtk_window_set_title (GTK_WINDOW (widget), _("Reset progress data"));
-		gtk_label_set_text (wg_label, _("This will DELETE all the progress data shown in the charts."));
+		tmp = g_strdup (tutor_get_type_name ());
+		for (msg = tmp; tmp[0] != '\0'; tmp++) tmp[0] = g_ascii_toupper (tmp[0]);
+		tmp = msg;
+		if (g_str_equal (tmp, "ADAPT") && gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_type"))) > 1)
+		{
+			g_free (tmp);
+			tmp = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (get_wg ("combobox_stat_type")));
+			msg = g_strdup_printf (_("This will delete the data of this chart. <%s>"), tmp);
+		}
+		else
+			msg = g_strdup_printf (_("This will delete the progress data of the module. (%s)"), tmp);
+		gtk_label_set_text (wg_label, msg);
+		g_free (tmp);
+		g_free (msg);
 	}
 	else
 	{
@@ -1109,6 +1235,8 @@ on_button_confirm_yes_clicked (GtkButton * button, gpointer user_data)
 {
 	gchar *file;
 	gchar *action;
+	gchar *msg;
+	gchar *tmp;
 	GtkWidget *wg;
 
 	wg = get_wg ("label_confirm_action");
@@ -1127,11 +1255,12 @@ on_button_confirm_yes_clicked (GtkButton * button, gpointer user_data)
 		keyb_remove_user_layout ();
 	}
 
-	else if (g_str_equal (action, "RESET"))
+	else if (g_str_equal (action, "RESET_ALL"))
 	{
 		file = g_build_filename (main_path_stats (), "stat_basic.txt", NULL);
 		g_unlink (file);
 		g_free (file);
+		basic_set_lesson (1);
 		file = g_build_filename (main_path_stats (), "stat_adapt.txt", NULL);
 		g_unlink (file);
 		g_free (file);
@@ -1144,13 +1273,64 @@ on_button_confirm_yes_clicked (GtkButton * button, gpointer user_data)
 		file = g_build_filename (main_path_stats (), "scores_fluid.txt", NULL);
 		g_unlink (file);
 		g_free (file);
-		accur_reset ();
 
-		basic_set_lesson (1);
+		accur_reset ();
+		accur_close ();
+		accur_init ();
 
 		on_combobox_stat_type_changed (NULL, NULL);
 	}
+	else if (g_str_equal (action, "RESET"))
+	{
+		tmp = g_strdup (tutor_get_type_name ());
+		for (msg = tmp; tmp[0] != '\0'; tmp++) tmp[0] = g_ascii_toupper (tmp[0]);
+		tmp = msg;
 
+		if (g_str_equal (tmp, "BASIC"))
+		{
+			file = g_build_filename (main_path_stats (), "stat_basic.txt", NULL);
+			g_unlink (file);
+			g_free (file);
+			basic_set_lesson (1);
+		}
+		else if (g_str_equal (tmp, "ADAPT"))
+		{
+			if (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_type"))) == 2)
+			{
+				accur_terror_reset ();
+				accur_close ();
+				accur_init ();
+			}
+			else if (gtk_combo_box_get_active (GTK_COMBO_BOX (get_wg ("combobox_stat_type"))) == 3)
+			{
+				accur_ttime_reset ();
+				accur_close ();
+				accur_init ();
+			}
+			else
+			{
+				file = g_build_filename (main_path_stats (), "stat_adapt.txt", NULL);
+				g_unlink (file);
+				g_free (file);
+			}
+		}
+		else if (g_str_equal (tmp, "VELO"))
+		{
+			file = g_build_filename (main_path_stats (), "stat_velo.txt", NULL);
+			g_unlink (file);
+			g_free (file);
+		}
+		else if (g_str_equal (tmp, "FLUID"))
+		{
+			file = g_build_filename (main_path_stats (), "stat_fluid.txt", NULL);
+			g_unlink (file);
+			g_free (file);
+			file = g_build_filename (main_path_stats (), "scores_fluid.txt", NULL);
+			g_unlink (file);
+			g_free (file);
+		}
+		on_combobox_stat_type_changed (NULL, NULL);
+	}
 	else
 		g_warning ("No valid action selected for 'yes' confirm-button: %s", action);
 
@@ -1320,9 +1500,14 @@ on_spinbutton_stat_lesson_value_changed (GtkSpinButton * spinbutton, gpointer us
 }
 
 G_MODULE_EXPORT void
-on_button_stat_reset_clicked (GtkButton * button, gpointer user_data)
+on_button_stat_reset_clicked (GtkButton * button, gpointer check)
 {
-	gtk_label_set_text (GTK_LABEL (get_wg ("label_confirm_action")), "RESET");
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)))
+		gtk_label_set_text (GTK_LABEL (get_wg ("label_confirm_action")), "RESET_ALL");
+	else
+		gtk_label_set_text (GTK_LABEL (get_wg ("label_confirm_action")), "RESET");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), FALSE);
+
 	gtk_widget_show (get_wg ("dialog_confirm"));
 }
 
@@ -1399,7 +1584,12 @@ on_button_other_apply_clicked (GtkButton *button, gpointer user_data)
 	gtk_tree_model_get (store, &iter, 0, &tmp_name, -1);
 
 	if (tutor_get_type () == TT_VELO)
-		velo_init_dict (tmp_name);
+	{
+		if (main_velo_txt ())
+			fluid_init_paragraph_list (tmp_name);
+		else
+			velo_init_dict (tmp_name);
+	}
 	else if (tutor_get_type () == TT_FLUID)
 		fluid_init_paragraph_list (tmp_name);
 
@@ -1424,7 +1614,7 @@ on_button_other_paste_clicked (GtkButton * button, gpointer user_data)
 		text = gtk_clipboard_wait_for_text (clipboard_2);
 	if (text == NULL)
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default());
 		g_message ("No text in the Clipboard");
 		return;
 	}
@@ -1433,10 +1623,17 @@ on_button_other_paste_clicked (GtkButton * button, gpointer user_data)
 	list = GTK_LIST_STORE ( gtk_tree_view_get_model (GTK_TREE_VIEW (wg)) );
 	if (tutor_get_type () == TT_VELO)
 	{
-		velo_text_write_to_file (text, TRUE);
-		tutor_load_list_other (".words", list);
+		if (main_velo_txt ())
+		{
+			fluid_text_write_to_file (text);
+			tutor_load_list_other (".paragraphs", list);
+		}
+		else
+		{
+			velo_text_write_to_file (text, TRUE);
+			tutor_load_list_other (".words", list);
+		}
 	}
-
 	else if (tutor_get_type () == TT_FLUID)
 	{
 		fluid_text_write_to_file (text);
@@ -1461,7 +1658,7 @@ on_button_other_remove_clicked (GtkButton * button, gpointer user_data)
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (get_wg ("treeview_other")));
 	if (gtk_tree_selection_get_selected (sel, &store, &iter) == FALSE)
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default());
 		return;
 	}
 
@@ -1469,7 +1666,7 @@ on_button_other_remove_clicked (GtkButton * button, gpointer user_data)
 
 	if (g_str_equal (tmp_str, OTHER_DEFAULT))
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default());
 		g_free (tmp_str);
 		return;
 	}
@@ -1478,7 +1675,10 @@ on_button_other_remove_clicked (GtkButton * button, gpointer user_data)
 	g_free (tmp_str);
 
 	if (tutor_get_type () == TT_VELO)
-		tmp_str = g_strconcat (tmp_name, ".words", NULL);
+		if (main_velo_txt ())
+			tmp_str = g_strconcat (tmp_name, ".paragraphs", NULL);
+		else
+			tmp_str = g_strconcat (tmp_name, ".words", NULL);
 	else if (tutor_get_type () == TT_FLUID)
 		tmp_str = g_strconcat (tmp_name, ".paragraphs", NULL);
 	g_unlink (tmp_str);
@@ -1592,7 +1792,10 @@ on_button_filechooser_open_clicked (GtkButton *button, gpointer user_data)
 	wg = get_wg ("treeview_other");
 	list = GTK_LIST_STORE ( gtk_tree_view_get_model (GTK_TREE_VIEW (wg)) );
 	if (tutor_get_type () == TT_VELO)
-		tutor_load_list_other (".words", list);
+		if (main_velo_txt ())
+			tutor_load_list_other (".paragraphs", list);
+		else
+			tutor_load_list_other (".words", list);
 	else if (tutor_get_type () == TT_FLUID)
 		tutor_load_list_other (".paragraphs", list);
 
