@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /* Functions to implement and manage the keyboard editing operations
  */
@@ -32,6 +33,9 @@
 extern gchar *KEYB_CUSTOM;
 extern gchar *KEYB_EDIT;
 
+//static GtkCssProvider *keyb_css = NULL;
+GtkCssProvider *keyb_css = NULL;
+
 static struct
 {
 	gchar *name;
@@ -209,26 +213,42 @@ keyb_create_virtual_keys ()
 	gchar *hlp;
 	GtkFixed *fix;
 	GdkRGBA color;
-	
-	/* Set color of space key
-	 */
-	if (main_preferences_exist ("colors", "key_5"))
-		hlp = main_preferences_get_string ("colors", "key_5");
-	else
-		hlp = g_strdup (KEYB_PURPLE);
-	gdk_rgba_parse (&color, hlp);
-	gtk_widget_override_background_color (get_wg ("but_space"), GTK_STATE_FLAG_NORMAL, &color);
-	g_free (hlp);
+	gchar *css_text;
+	gchar *tmp;
+	gchar *tmp2;
+	gchar chcd;
+	GtkStyleContext *sc;
 
-	/* Set text color of keys
+	/* Set color of keys
 	 */
 	if (main_preferences_exist ("colors", "key_fg"))
 		hlp = main_preferences_get_string ("colors", "key_fg");
 	else
 		hlp = g_strdup (KEYB_BLACK);
-	gdk_rgba_parse (&color, hlp);
+	if (keyb_css == NULL)
+	{
+		keyb_css = gtk_css_provider_new ();
+		css_text = g_strdup ("");
+		for (chcd = '1'; chcd <= '9'; chcd++)
+		{
+			tmp = g_strdup_printf (".key-but%c {background-image: none; background-color: %s; color: %s} "
+					       ".key-but%c:hover {background-image: none; background-color: white; color: black} "
+					       ".key-but%c:active {background-image: none; background-color: black; color: white} ",
+				               chcd, hints_color_from_charcode (chcd), hlp, chcd, chcd);
+			css_text = g_strdup_printf ("%s%s", css_text, tmp); 
+			g_free (tmp);
+		}
+		gtk_css_provider_load_from_data (keyb_css, css_text, -1, NULL);
+		g_free (css_text);
+	}
 	g_free (hlp);
 
+	/* Set space key
+	 */
+	sc = gtk_widget_get_style_context (get_wg ("but_space"));
+	gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (keyb_css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gtk_style_context_add_class (sc, "key-but5");
+
 	/* Create and position buttons and labels
 	 */
 	fix = GTK_FIXED (get_wg ("fixed_keyboard"));
@@ -244,8 +264,6 @@ keyb_create_virtual_keys ()
   			g_signal_connect_after ((gpointer) keyb.but[i][j], "grab-focus",
 				       	G_CALLBACK (on_virtual_key_grab_focus), NULL);
 			keyb.lab[i][j] = gtk_label_new ("0");
-			gtk_widget_override_color (keyb.lab[i][j], GTK_STATE_FLAG_NORMAL, &color);
-			gtk_widget_override_color (keyb.lab[i][j], GTK_STATE_FLAG_PRELIGHT, &color);
 			gtk_container_add (GTK_CONTAINER (keyb.but[i][j]), keyb.lab[i][j]);
 
 			if (i > 0)
@@ -1059,7 +1077,7 @@ keyb_update_combos (gchar *cmb_country, gchar *cmb_variant)
 void
 keyb_intro_step_next ()
 {
-	if (keyb.intro_step < 6)
+	if (keyb.intro_step < 7)
 		keyb_intro_step (++keyb.intro_step);
 }
 
@@ -1090,6 +1108,7 @@ keyb_intro_step (gint step)
 	static gchar *intro08 = NULL;
 	static gchar *intro09 = NULL;
 	static gchar *intro10 = NULL;
+	static gchar *intro11 = NULL;
 
 	if (intro01 == NULL)
 	{
@@ -1124,6 +1143,10 @@ keyb_intro_step (gint step)
 	"After memorizing this relationship, you can relax the previous rule some, "
 	"so that you can attain greater speed while typing."));
 		intro10 = g_strdup (_(
+	"The shift keys are used for capital letters and for some symbols. "
+	"To get a shifted key input you should first use the small finger of the opposite hand. "
+	"Just keep it in the closest Shift while reaching the target key with the other hand."));
+		intro11 = g_strdup (_(
 	"You should be prepared to start training with the basic course. "
 	"It will take effort and patience to be successful as a typist. "
 	"We trust you have these and look forward to your success!"));
@@ -1139,8 +1162,6 @@ keyb_intro_step (gint step)
 	switch (step)
 	{
 	case 0:	/* Recommendations */
-		gtk_label_set_text (tx1, intro01);
-		gtk_label_set_text (tx2, "");
 		gtk_label_set_text (tit, _("To position the hands"));
         	gtk_text_buffer_set_text (buffer, intro01, -1);
 		gtk_widget_grab_focus (get_wg ("button_keyboard_next"));
@@ -1149,8 +1170,6 @@ keyb_intro_step (gint step)
 		break;
 	case 1: /* Index fingers */
 		gtk_label_set_text (tit, intro00);
-		gtk_label_set_text (tx1, intro02);
-		gtk_label_set_text (tx2, intro03);
         	gtk_text_buffer_set_text (buffer, intro02, -1);
         	gtk_text_buffer_insert_at_cursor (buffer, "\n", -1);
         	gtk_text_buffer_insert_at_cursor (buffer, intro03, -1);
@@ -1161,8 +1180,6 @@ keyb_intro_step (gint step)
 		break;
 	case 2: /* Fingers beside index */
 		gtk_label_set_text (tit, intro00);
-		gtk_label_set_text (tx1, intro04);
-		gtk_label_set_text (tx2, "");
         	gtk_text_buffer_set_text (buffer, intro04, -1);
 		keyb_set_sensitive (FALSE);
 		gtk_widget_set_sensitive (keyb.but[2][0], TRUE);
@@ -1175,8 +1192,6 @@ keyb_intro_step (gint step)
 		break;
 	case 3: /* Thumbs and wrists */
 		gtk_label_set_text (tit, intro00);
-		gtk_label_set_text (tx1, intro05);
-		gtk_label_set_text (tx2, intro06);
         	gtk_text_buffer_set_text (buffer, intro05, -1);
         	gtk_text_buffer_insert_at_cursor (buffer, "\n", -1);
         	gtk_text_buffer_insert_at_cursor (buffer, intro06, -1);
@@ -1188,8 +1203,6 @@ keyb_intro_step (gint step)
 		break;
 	case 4: /* The home position */
 		gtk_label_set_text (tit, intro00);
-		gtk_label_set_text (tx1, intro07);
-		gtk_label_set_text (tx2, "");
         	gtk_text_buffer_set_text (buffer, intro07, -1);
 		keyb_set_sensitive (FALSE);
 		gtk_widget_set_sensitive (keyb.but[2][0], TRUE);
@@ -1204,8 +1217,6 @@ keyb_intro_step (gint step)
 		break;
 	case 5: /* Reaching keys */
 		gtk_label_set_text (tit, intro00);
-		gtk_label_set_text (tx1, intro08);
-		gtk_label_set_text (tx2, intro09);
         	gtk_text_buffer_set_text (buffer, intro08, -1);
         	gtk_text_buffer_insert_at_cursor (buffer, "\n", -1);
         	gtk_text_buffer_insert_at_cursor (buffer, intro09, -1);
@@ -1223,19 +1234,30 @@ keyb_intro_step (gint step)
 		gtk_widget_set_sensitive (get_wg ("toggle_shift2"), FALSE);
 		hints_demo_fingers (1000/5);
 		break;
-	case 6: /* Final words */
-		gtk_label_set_text (tx1, intro10);
-		gtk_label_set_text (tx2, "");
-		gtk_label_set_text (tit, _("Go ahead!"));
+	case 6: /* Shift key */
+		gtk_label_set_text (tit, intro00);
         	gtk_text_buffer_set_text (buffer, intro10, -1);
+		keyb_set_sensitive (FALSE);
+		gtk_widget_set_sensitive (keyb.but[0][0], TRUE);
+		gtk_widget_set_sensitive (keyb.but[0][13], TRUE);
+		gtk_widget_set_sensitive (get_wg ("toggle_shift1"), TRUE);
+		gtk_widget_set_sensitive (get_wg ("toggle_shift2"), TRUE);
+		hints_demo_fingers (1000/2);
+		break;
+	case 7: /* Final words */
+		gtk_label_set_text (tit, _("Go ahead!"));
+        	gtk_text_buffer_set_text (buffer, intro11, -1);
 		gtk_widget_grab_focus (get_wg ("button_keyboard_close"));
 		keyb_set_sensitive (TRUE);
 		hints_demo_fingers (0);
 		break;
 	default:
+		g_free (intro00);
+		intro00 = g_strdup (_("Click on any key to see which finger you must use:"));
 		gtk_label_set_text (tit, _("Relation between fingers and keys"));
 		gtk_label_set_text (tx1, _("Click on any key to see which finger you must use:"));
 		gtk_label_set_text (tx2, "");
+        	gtk_text_buffer_set_text (buffer, intro00, -1);
 		keyb_set_sensitive (TRUE);
 		hints_demo_fingers (0);
 	}
@@ -1251,12 +1273,12 @@ keyb_intro_step (gint step)
 	else
 		gtk_widget_set_sensitive (get_wg ("button_keyboard_previous"), TRUE);
 
-	if (step == 6)
+	if (step == 7)
 		gtk_widget_set_sensitive (get_wg ("button_keyboard_next"), FALSE);
 	else
 		gtk_widget_set_sensitive (get_wg ("button_keyboard_next"), TRUE);
 
-	if (step >= 0 && step <= 6)
+	if (step >= 0 && step <= 7)
 		keyb.intro_step = step;
 	else
 		keyb.intro_step = 0;
@@ -1556,13 +1578,13 @@ keyb_change_key (gunichar real_key)
 gchar *
 keyb_get_utf8_paragraph_symbol ()
 {
-	static gchar parsym[7];
+	static gchar parsym[7] = {0, 0, 0, 0, 0, 0, 0};
 	static gboolean is_initialized = FALSE;
 
 	if (is_initialized == FALSE)
 	{
 		is_initialized = TRUE;
-		parsym[g_unichar_to_utf8 (UPSYM, parsym)] = '\0';
+		g_unichar_to_utf8 (UPSYM, parsym);
 	}
 	return (parsym);
 }
@@ -1608,6 +1630,7 @@ hints_init ()
 	else
 		g_warning ("couldn't open the file:\n %s", tmp_name);
 	g_free (tmp_name);
+
 }
 
 gchar *
@@ -1757,6 +1780,8 @@ hints_set_colors ()
 	gint i, j;
 	gint j_max;
 	GdkRGBA color;
+	GtkStyleContext *sc;
+	gchar *tmp;
 
 	if (hints_is_initialized == FALSE)
 	{
@@ -1769,8 +1794,11 @@ hints_set_colors ()
 		j_max = KEY_LINE_LEN - (i == 0 ? 1 : (i == 1 ? 2 : 3));
 		for (j = 0; j < j_max; j++)
 		{
-			gdk_rgba_parse (&color, hints_color_from_charcode (hints[i][j]));
-			gtk_widget_override_background_color (keyb.but[i][j], GTK_STATE_FLAG_NORMAL, &color);
+			sc = gtk_widget_get_style_context (keyb.but[i][j]);
+			gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (keyb_css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+			tmp = g_strdup_printf ("key-but%c", hints[i][j]);
+			gtk_style_context_add_class (sc, tmp);
+			g_free (tmp);
 		}
 	}
 }
@@ -1909,3 +1937,4 @@ hints_finger_name_from_char (gunichar uch)
 
 	return (g_strdup (" "));
 }
+
