@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * Charts management
@@ -21,7 +22,6 @@
 #include <glib/gstdio.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <gtkdatabox_typedefs.h>
 #include <gtkdatabox.h>
 #include <gtkdatabox_grid.h>
 #include <gtkdatabox_lines.h>
@@ -90,6 +90,8 @@ gchar language[DATA_POINTS+1][80+1];
 glong n_points;
 gint plot_type; /* used to communicate the plotting type, for updating the cursor marker, etc */
 
+gint lesson_n;
+
 /*******************************************************************************
  * Interface functions
  */
@@ -101,31 +103,13 @@ plot_get_databox ()
 
 /*******************************************************************************
  * Private functions
-
-static void
-plot_clip_data (gint i)
-{
-		if (accur[i] > 100)
-			accur[i] = 100.1;
-		if (accur[i] < 60)
-			accur[i] = 59.9;
-		if (velo[i] > 100)
-			velo[i] = 100.1;
-		if (velo[i] < 0)
-			velo[i] = -0.1;
-		if (fluid[i] > 100)
-			fluid[i] = 100.1;
-		if (fluid[i] < 0)
-			fluid[i] = -0.1;
-		if (score[i] < 0)
-			score[i] = -0.1;
-}
  */
 
 static void
 plot_error_frequencies ()
 {
 	gint i;
+	gchar *tcolor;
 	GdkRGBA color;
 	GdkRGBA color_black;
 	GtkDatabox *box;
@@ -154,12 +138,12 @@ plot_error_frequencies ()
 	if (plot.lim.y[1] < 1)
 		plot.lim.y[1] = 1.05;
 
-	/* White background
-	 */
-	gdk_rgba_parse (&color, "#ffffff");
-	gtk_widget_override_background_color (plot.databox, GTK_STATE_FLAG_NORMAL, &color);
-	 
-	gdk_rgba_parse (&color_black, "#000000");
+	/* Plot color */
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+		tcolor = main_altcolor_get_string ("colors", "char_untouched_fg");
+	else
+		tcolor = main_preferences_get_string ("colors", "char_untouched_fg");
+	gdk_rgba_parse (&color_black, tcolor);
 	gdk_rgba_parse (&color, PLOT_GREEN_2);
 
 	/* Point limits */
@@ -191,6 +175,7 @@ static void
 plot_touch_times ()
 {
 	gint i;
+	gchar *tcolor;
 	GdkRGBA color;
 	GdkRGBA color_black;
 	GtkDatabox *box;
@@ -219,12 +204,12 @@ plot_touch_times ()
 	if (plot.lim.y[1] < 0.01)
 		plot.lim.y[1] = 0.0105;
 
-	/* White background
-	 */
-	gdk_rgba_parse (&color, "#ffffff");
-	gtk_widget_override_background_color (plot.databox, GTK_STATE_FLAG_NORMAL, &color);
-	 
-	gdk_rgba_parse (&color_black, "#000000");
+	/* Plot color */
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+		tcolor = main_altcolor_get_string ("colors", "char_untouched_fg");
+	else
+		tcolor = main_preferences_get_string ("colors", "char_untouched_fg");
+	gdk_rgba_parse (&color_black, tcolor);
 	gdk_rgba_parse (&color, PLOT_PURPLE);
 
 	/* Point limits */
@@ -259,6 +244,9 @@ void
 plot_initialize ()
 {
 	static gboolean inited = FALSE;
+	static GtkCssProvider *css;
+	GtkStyleContext *sc;
+	gchar *tcolor;
 	gint i;
 
 	if (inited)
@@ -276,6 +264,19 @@ plot_initialize ()
 	gtk_container_add (GTK_CONTAINER (get_wg ("frame_stat")), plot.gtkgrid);
 	g_signal_connect (G_OBJECT (plot.databox), "motion_notify_event", G_CALLBACK (on_databox_hovered), NULL);
 
+	/* Set background not working: FIXME
+	css = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (css, ".plot_bg {background-color: white;}", -1, NULL);
+	sc = gtk_widget_get_style_context (plot.databox);
+	gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER (css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gtk_style_context_add_class (sc, "plot_bg");
+	 */
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+		tcolor = main_altcolor_get_string ("colors", "text_intro_bg");
+	else
+		tcolor = main_preferences_get_string ("colors", "text_intro_bg");
+	gtk_databox_background (tcolor); // This is a hack!
+
 	/* Y labels
 	 */
 	for (i = 0; i < MAX_Y_LABELS; i++)
@@ -291,16 +292,16 @@ plot_initialize ()
  * Plots the statistics
  */
 void
-plot_draw_chart (gint field)
+plot_draw_chart (gint pltype)
 {
 	gint i, len;
-	gint lesson_n;
 	gchar *kb_name;
 	gchar *tmp_name;
 	gchar *tmp_locale;
 	gchar *tmp_lang;
 	gchar tmp_str[2000];
 	FILE *fh;
+	gchar *tcolor;
 	GdkRGBA color, color2, color3;
 	GdkRGBA color_black;
 	GtkDatabox *box;
@@ -313,18 +314,18 @@ plot_draw_chart (gint field)
 	gtk_widget_hide (plot.gtkgrid);
 
 	/* Set plot type for external reference */
-	plot_type = field;
+	plot_type = pltype;
 
 	/* Error frequencies or touch times
 	 */
 	gtk_widget_set_tooltip_text (get_wg ("entry_stat_x"), _("Character"));
 	gtk_widget_hide (get_wg ("box_grid_label_y"));
-	if (field == 6)
+	if (pltype == 6)
 	{
 		plot_error_frequencies ();
 		return;
 	}
-	else if (field == 7)
+	else if (pltype == 7)
 	{
 		plot_touch_times ();
 		return;
@@ -349,7 +350,7 @@ plot_draw_chart (gint field)
 
 	/* Get the file name
 	 */
-	if (field < 4)
+	if (pltype < 4)
 		tmp_name = g_strconcat (main_path_stats (), G_DIR_SEPARATOR_S "stat_",
 			       	tutor_get_type_name (), ".txt", NULL);
 	else
@@ -372,7 +373,7 @@ plot_draw_chart (gint field)
 	if (tmp_locale != NULL)
 		setlocale (LC_NUMERIC, "C");
 
-	/* Keyboard names are compared without spaces (may be needed for custom files)
+	/* Keyboard names must be compared without spaces (may appear in custom files)
 	 */
 	kb_name = g_strdup (keyb_get_name ());
 	for (i=0; kb_name[i]; i++)
@@ -387,6 +388,7 @@ plot_draw_chart (gint field)
 		fclose (fh);
 		return;
 	}
+	g_free (tmp_name);
 
 	/* Read the first DATA_POINTS points
 	 */
@@ -399,7 +401,7 @@ plot_draw_chart (gint field)
 		gchar *lang_extra;
 
 		language[i][0] = '\0';
-		if (field < 4)
+		if (pltype < 4)
 		{
 			itens = fscanf (fh, "%f%f%f%s%s%s\t", &accur[i], &velo[i], &fluid[i],
 				       	date[i], hour[i], lesson[i]);
@@ -414,13 +416,12 @@ plot_draw_chart (gint field)
 		if (language[i][len = (strlen(language[i])-1)] == '\n')
 			language[i][len] = '\0';
 
-		//plot_clip_data (i);
-
 		if (tutor_get_type () == TT_BASIC)
 		{
 			if (g_ascii_strtoll (lesson[i], NULL, 10) != lesson_n)
-				continue;
-			if (strcmp (language[i], kb_name) != 0)
+				if (lesson_n != 0)
+					continue;
+			if (strcmp (language[i], kb_name) != 0) // in BASIC, language field is the keyboard.
 				continue;
 		}
 		if (tutor_get_type () == TT_ADAPT && strcmp (lesson[i], kb_name) != 0)
@@ -429,7 +430,7 @@ plot_draw_chart (gint field)
 			continue;
 		if (tutor_get_type () == TT_FLUID && strcmp (language[i], tmp_lang) != 0)
 			continue;
-		switch (field)
+		switch (pltype)
 		{
 		case 1:
 			plot.data.y[i] = accur[i];
@@ -456,7 +457,7 @@ plot_draw_chart (gint field)
 		gchar *lang_extra;
 
 		language[i][0] = '\0';
-		if (field < 4)
+		if (pltype < 4)
 		{
 			itens = fscanf (fh, "%f%f%f%s%s%s\t", &accur[i], &velo[i], &fluid[i],
 				       	date[i], hour[i], lesson[i]);
@@ -471,12 +472,11 @@ plot_draw_chart (gint field)
 		if (language[i][len = (strlen(language[i])-1)] == '\n')
 			language[i][len] = '\0';
 
-		//plot_clip_data (i);
-
 		if (tutor_get_type () == TT_BASIC)
 		{
 			if (g_ascii_strtoll (lesson[i], NULL, 10) != lesson_n)
-				continue;
+				if (lesson_n != 0)
+					continue;
 			if (strcmp (language[i], kb_name) != 0)
 				continue;
 		}
@@ -495,7 +495,7 @@ plot_draw_chart (gint field)
 		strcpy (date[i], date[i + 1]);
 		strcpy (hour[i], hour[i + 1]);
 
-		switch (field)
+		switch (pltype)
 		{
 		case 1:
 			plot.data.y[i] = accur[i + 1];
@@ -514,7 +514,6 @@ plot_draw_chart (gint field)
 	}
 	fclose (fh);
 	g_free (kb_name);
-	g_free (tmp_name);
 
 	/* Coming back to the right locale
 	 */
@@ -522,20 +521,11 @@ plot_draw_chart (gint field)
 		setlocale (LC_NUMERIC, tmp_locale);
 	g_free (tmp_locale);
 
-	/* Set apropriate background
-	 */
 	if (i == 0)
 	{
 		g_message ("no valid data to plot.");
-		gdk_rgba_parse (&color, "#ccccce");
-		gtk_widget_override_background_color (plot.databox, GTK_STATE_FLAG_NORMAL, &color);
 		return;
 	}
-	else
-	{
-		gdk_rgba_parse (&color, "#ffffff");
-		gtk_widget_override_background_color (plot.databox, GTK_STATE_FLAG_NORMAL, &color);
-	}
 	n_points = i;
 
 	/* Format the chart
@@ -547,8 +537,12 @@ plot_draw_chart (gint field)
 	plot.lim.y[0] = 0;
 	plot.lim.y[1] = 100;
 	 
-	gdk_rgba_parse (&color_black, "#000000");
-	switch (field)
+	if (main_altcolor_get_boolean ("colors", "altcolor"))
+		tcolor = main_altcolor_get_string ("colors", "char_untouched_fg");
+	else
+		tcolor = main_preferences_get_string ("colors", "char_untouched_fg");
+	gdk_rgba_parse (&color_black, tcolor);
+	switch (pltype)
 	{
 	case 1:
 		plot.lim.y[0] = 60;
@@ -609,33 +603,35 @@ plot_draw_chart (gint field)
 	gtk_databox_graph_add (box, plot.line_outter);
 
 	/* Goal limit */
-	gdk_rgba_parse (&color3, "#999999");
+	gdk_rgba_parse (&color3, "#808080");
 	plot.line_goal = gtk_databox_lines_new (2, plot.goal.x, plot.goal.y, &color3, 1);
 	gtk_databox_graph_add (box, plot.line_goal);
 
 	/* Grid and y labels */
-	gdk_rgba_parse (&color, "#dddddd");
-	if (field == 1) /* Correctness (%) */
+	gdk_rgba_parse (&color, "#bbbbbb");
+	if (pltype == 1) /* Correctness (%) */
 	{
 		plot.grid = gtk_databox_grid_new (3, 3, &color, 1);
 		for (i = 0; i < 5; i++)
 		{
 			g_sprintf (tmp_str, "%u", 100 - 10*i);
 			gtk_label_set_text (GTK_LABEL (plot.label_y[i]), tmp_str);
-			gtk_misc_set_alignment (GTK_MISC (plot.label_y[i]), 1.0, i / 4.0);
+			gtk_label_set_xalign (GTK_LABEL (plot.label_y[i]), 0.9);
+			gtk_label_set_yalign (GTK_LABEL (plot.label_y[i]), i / 4.0);
 			gtk_widget_show (plot.label_y[i]);
 		}
 		for (; i < MAX_Y_LABELS; i++)
 			gtk_widget_hide (plot.label_y[i]);
 	}
-	else if (field == 2) /* Speed (WPM) */
+	else if (pltype == 2) /* Speed (WPM) */
 	{
 		plot.grid = gtk_databox_grid_new (11, 3, &color, 1);
 		for (i = 0; i < 13; i++)
 		{
 			g_sprintf (tmp_str, "%u", 120 - 10*i);
 			gtk_label_set_text (GTK_LABEL (plot.label_y[i]), tmp_str);
-			gtk_misc_set_alignment (GTK_MISC (plot.label_y[i]), 1.0, i / 12.0);
+			gtk_label_set_xalign (GTK_LABEL (plot.label_y[i]), 0.9);
+			gtk_label_set_yalign (GTK_LABEL (plot.label_y[i]), i / 12.0);
 			gtk_widget_show (plot.label_y[i]);
 		}
 	}
@@ -644,12 +640,13 @@ plot_draw_chart (gint field)
 		plot.grid = gtk_databox_grid_new (9, 3, &color, 1);
 		for (i = 0; i < 11; i++)
 		{
-			if (field == 3)
+			if (pltype == 3)
 				g_sprintf (tmp_str, "%u", 100 - 10*i);
 			else
 				g_sprintf (tmp_str, "%u", 10 - i);
 			gtk_label_set_text (GTK_LABEL (plot.label_y[i]), tmp_str);
-			gtk_misc_set_alignment (GTK_MISC (plot.label_y[i]), 1.0, i / 10.0);
+			gtk_label_set_xalign (GTK_LABEL (plot.label_y[i]), 0.9);
+			gtk_label_set_yalign (GTK_LABEL (plot.label_y[i]), i / 10.0);
 			gtk_widget_show (plot.label_y[i]);
 		}
 		for (; i < MAX_Y_LABELS; i++)
@@ -669,6 +666,7 @@ plot_pointer_update (gdouble x)
 	glong n = 0;
 	gchar *xstr;
 	gchar *ystr;
+	gchar *tmp;
 	GtkDatabox *box;
 	gint width;
 
@@ -696,7 +694,10 @@ plot_pointer_update (gdouble x)
 	{
 		if (plot_type < 6)
 		{
-			xstr = g_strdup_printf ("%s - %s", date[n], hour[n]);
+			if (tutor_get_type () == TT_BASIC && lesson_n == 0)
+				xstr = g_strdup_printf ("%s - %s [%s]", date[n], hour[n], lesson[n]);
+			else
+				xstr = g_strdup_printf ("%s - %s", date[n], hour[n]);
 			ystr = g_strdup_printf ("%.2f", plot.data.y[n]);
 		}
 		else if (plot_type == 6)
