@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * Contest for fluidness performance
@@ -252,48 +253,43 @@ top10_delete_stat (gint i, gboolean locally)
 	top10_clean_stat (9, locally);
 }
 
-gfloat
-top10_calc_score_old (Statistics * stat)
-{
-	gfloat score;
-	// Don't touch this anymore!
-	score = (6 * stat->velo + 3 * stat->accur + stat->fluid) / 100;
-	return (score);
-}
-
 gfloat
 top10_calc_score (Statistics * stat)
 {
 	gfloat score;
-	// Don't touch this anymore! (touched: new_WPM = 1.2 old_WPM)
-	score = (6 * stat->velo / 1.2 + 3 * stat->accur + stat->fluid) / 100;
+	/* Weighs: 6 ... 3 ... 1 ==> 10 */
+	score = (6.0 * stat->velo / 1.2 + 3.0 * stat->accur + 1.0 * stat->fluid) / 100;
 	return (score);
 }
 
 gboolean
 top10_validate_stat (Statistics * stat)
 {
-
-	if (stat->score > 10.0 || stat->score < 0.0)
+	if (stat->velo >= 222.222)
 	{
-		g_message ("Invalid score (%g): it must be between 0 and 10", stat->score);
-		return (FALSE);
-	}
-
-	if ((stat->score - top10_calc_score (stat) > 1.0e-5) && (stat->score - top10_calc_score_old (stat) > 1.0e-5))
-	{
-		g_message ("Invalid score (%g): did you try to edit the scores file?", stat->score);
+		g_message ("Invalid speed (%g): too much fast for a human on a simple keyboard.", stat->score);
 		return (FALSE);
 	}
 
 	if (stat->fluid > 95)
 	{
-		g_message
-			("Invalid score (fluidness = %g): robots shouldn't compete with humans...",
+		g_message ("Invalid fluidity (%g): robots shouldn't compete with humans...",
 			 stat->fluid);
 		return (FALSE);
 	}
 
+	if (stat->score > 17.3333 || stat->score < 0.0)
+	{
+		g_message ("Invalid score (%g): it must be between 0 and 17.3333", stat->score);
+		return (FALSE);
+	}
+
+	if (stat->score - top10_calc_score (stat) > 1.0e-5)
+	{
+		g_message ("Invalid score (%g): do not edit the scorings file.", stat->score);
+		return (FALSE);
+	}
+
 	return (TRUE);
 }
 
@@ -319,7 +315,7 @@ top10_get_score_file (gboolean locally, gint lang)
 	return (ksc);
 }
 
-static void
+static gboolean
 top10_merge_stats_from_file (gchar * file)
 {
 	gint i, j;
@@ -329,7 +325,8 @@ top10_merge_stats_from_file (gchar * file)
 
 	fh = g_fopen (file, "r");
 	if (fh == NULL)
-		return;
+		return FALSE;
+	g_message ("1: %s", file);
 
 	for (i = 0; i < 10; i++)
 	{
@@ -427,6 +424,7 @@ top10_merge_stats_from_file (gchar * file)
 		top10_compare_insert_stat (&top10, TRUE);
 	}
 	fclose (fh);
+	return TRUE;
 }
 
 gboolean
@@ -606,11 +604,7 @@ top10_read_stats (gboolean locally, gint lang)
 				continue;
 
 			tmp = g_build_filename ("/home", uname, stat_dir, "ksc", local_ksc, NULL);
-			if (g_file_test (tmp, G_FILE_TEST_IS_REGULAR))
-			{
-				top10_merge_stats_from_file (tmp);
-				success = TRUE;
-			}
+			success = success ? TRUE : top10_merge_stats_from_file (tmp);
 			g_free (tmp);
 		}
 		g_dir_close (home);
@@ -740,7 +734,7 @@ top10_show_stats (gboolean locally)
 		g_free (tmp);
 		tmp = g_strdup ("en");
 	}
-	url = g_strdup_printf ("http://" DOWNHOST "/%c%c/", tmp[0], tmp[1]);
+	url = g_strdup_printf ("https://" DOWNHOST "/%c%c/", tmp[0], tmp[1]);
 	gtk_widget_set_tooltip_text (get_wg ("button_top10_go_www"), url);
 	g_free (tmp);
 	g_free (url);
@@ -838,7 +832,7 @@ top10_global_update (gpointer data)
 		g_mkdir_with_parents (main_path_score (), DIR_PERM);
 	tmp = g_build_filename (main_path_score (), ksc, NULL);
 	fail = TRUE;
-	command = g_strdup_printf ("http://%s/%s", host, ksc);
+	command = g_strdup_printf ("https://%s/%s", host, ksc);
 	if ( (fh = g_fopen (tmp, "wb")) )
 	{
 		/*
@@ -849,7 +843,9 @@ top10_global_update (gpointer data)
 		curl_easy_setopt (curl, CURLOPT_LOW_SPEED_TIME, LOW_SPEED_TIME);
 		curl_easy_setopt (curl, CURLOPT_URL, command);
 		curl_easy_setopt (curl, CURLOPT_WRITEDATA, fh);
+		curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, 0L);
 		fail = curl_easy_perform (curl);
+		if (fail) g_message (curl_easy_strerror (fail));
 		fclose (fh);
 	}
 	curl_easy_cleanup (curl);
@@ -886,6 +882,7 @@ top10_global_publish (gpointer data)
 {
 	gint i;
 	gboolean success;
+	gboolean fail;
 	gchar *tmp;
 	gchar *ksc;
 	gchar *host;
@@ -894,7 +891,7 @@ top10_global_publish (gpointer data)
 	gchar *url;
 	GtkImage *img;
 	FILE *fh, *fh2;
-	struct stat fs;
+	GStatBuf fs;
 	CURL *curl;
 	
 	fs.st_size = 0;
@@ -940,8 +937,13 @@ top10_global_publish (gpointer data)
 		g_free (username);
 		username = g_strdup (g_get_user_name ());
 	}
+	if (strlen (username) == 0)
+	{
+		g_free (username);
+		username = g_strdup ("0-0");
+	}
 	ksc = g_strdup_printf ("%s_%s_%c%c.ksc", username, g_get_host_name (), tmp[0], tmp[1]);
-	url = g_strdup_printf ("http://%s?dosiernomo=%s&lingvo=%c%c", host, ksc, tmp[0], tmp[1]);
+	url = g_strdup_printf ("https://%s?dosiernomo=%s&lingvo=%c%c", host, ksc, tmp[0], tmp[1]);
 	g_free (username);
 	g_free (host);
 	g_free (ksc);
@@ -970,8 +972,9 @@ top10_global_publish (gpointer data)
 		g_free (tmp);
 		curl_easy_setopt (curl, CURLOPT_READDATA, fh);
 		curl_easy_setopt (curl, CURLOPT_WRITEDATA, fh2);
-		if (curl_easy_perform (curl))
-			g_message ("HTTP upload failed!");
+		curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, 0L);
+		if (fail = curl_easy_perform (curl))
+			g_message ("HTTPS upload failed: %s", curl_easy_strerror(fail));
 		else
 			success = TRUE;
 		fclose (fh);
