@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * This definition contains the set of languages configured for
@@ -35,7 +36,7 @@
 "English (C) [qwerty_us]\n" \
 "English UK (en_GB) [qwerty_uk]\n" \
 "Esperanto (eo) [dvorak_eo_eurokeys]\n" \
-"Español; Castellano (es) [qwerty_es]\n" \
+"Español (es) [qwerty_es]\n" \
 "Euskara (eu) [qwerty_es]\n" \
 "Français (fr) [azerty_fr]\n" \
 "Galego (gl) [qwerty_es]\n" \
@@ -48,10 +49,11 @@
 "Nederlands (nl) [qwerty_us]\n" \
 "ਪੰਜਾਬੀ (pa) [gumurkhi_in_jehlum]\n" \
 "Polski (pl) [qwerty_pl_us]\n" \
-"Português (pt_BR) [qwerty_br_abnt2]\n" \
+"Português BR (pt_BR) [qwerty_br_abnt2]\n" \
+"Português PT (pt_PT) [qwerty_pt]\n" \
 "Русский (ru) [jtsuken_ru]\n" \
 "Suomen kieli (fi) [qwerty_se]\n"\
-"Slovenščina (sl) [qwertz_sl]\n" \
+"Slovenščina (sl) [qwertz_si]\n" \
 "Српски (sr) [qwertz_rs]\n" \
 "Svenska (sv) [qwerty_se]\n" \
 "తెలుగు (te) [qwerty_us]\n" \
