@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * Shared tutor window tasks
@@ -38,7 +39,9 @@
 #include "top10.h"
 #include "tutor.h"
 
-#define MAX_TOUCH_TICS 10000
+extern GtkCssProvider *keyb_css;
+
+#define MAX_TOUCH_TICS 4000
 struct
 {
 	TutorType type;
@@ -440,6 +443,7 @@ tutor_update_intro ()
 	GtkAdjustment *scroll;
 	GtkTextIter start;
 	GtkTextIter end;
+	GtkStyleContext *sc;
 
 	if (tutor.type == TT_BASIC)
 	{
@@ -467,11 +471,21 @@ tutor_update_intro ()
 	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "lesson_font", &start, &end);
 	gtk_text_buffer_apply_tag_by_name (gtk_text_view_get_buffer (wg_text), "text_intro", &start, &end);
 
+	/*
+	 * Apply tutor background color and font to the intro 
+	 * FIXME: update deprecated function for background color
+	 */
 	if (main_preferences_exist ("colors", "text_intro_bg"))
 		color_bg = main_preferences_get_string ("colors", "text_intro_bg");
 	else
 		color_bg = g_strdup (TUTOR_WHITE);
-	gdk_rgba_parse (&color, color_bg);
+	/* Check if altcolor applies */
+	if (main_altcolor_get_boolean ("colors", "altcolor") == TRUE)
+	{
+		g_free (color_bg);
+		color_bg = main_altcolor_get_string ("colors", "text_intro_bg");
+	}
+	gdk_rgba_parse (&color, color_bg); 
 	gtk_widget_override_background_color (get_wg ("text_tutor"), GTK_STATE_FLAG_INSENSITIVE, &color);
 	g_free (color_bg);
 
@@ -529,7 +543,10 @@ tutor_update_start ()
 		adapt_draw_random_pattern ();
 		break;
 	case TT_VELO:
-		velo_draw_random_words ();
+		if (main_velo_txt ())
+			fluid_draw_random_paragraphs ();
+		else
+			velo_draw_random_words ();
 		break;
 	case TT_FLUID:
 		fluid_draw_random_paragraphs ();
@@ -537,20 +554,27 @@ tutor_update_start ()
 
 	/*
 	 * Apply tutor background color and font to the text
+	 * FIXME: update deprecated function for background color
 	 */
 	if (main_preferences_exist ("colors", "char_untouched_bg"))
 		color_bg = main_preferences_get_string ("colors", "char_untouched_bg");
 	else
 		color_bg = g_strdup (TUTOR_CREAM);
+	/* Check if altcolor applies */
+	if (main_altcolor_get_boolean ("colors", "altcolor") == TRUE)
+	{
+		g_free (color_bg);
+		color_bg = main_altcolor_get_string ("colors", "char_untouched_bg");
+	}
 	gdk_rgba_parse (&color, color_bg);
 	gtk_widget_override_background_color (get_wg ("text_tutor"), GTK_STATE_FLAG_INSENSITIVE, &color);
 	g_free (color_bg);
 
 	gtk_text_buffer_get_bounds (buf, &start, &end);
-	gtk_text_iter_backward_char (&end);
 	gtk_text_buffer_apply_tag_by_name (buf, "lesson_font", &start, &end);
+	gtk_text_buffer_apply_tag_by_name (buf, "char_untouched", &start, &end);
 
-	/* Trying to minimize automatic wrapping because of cursor blinking:
+	/* Trying to minimize wrapping toggle because of cursor blinking:
 	*/
 	end = start;
 	while (gtk_text_iter_forward_word_end (&end))
@@ -772,7 +796,6 @@ tutor_eval_forward (gunichar user_chr)
 			if (tutor.type != TT_BASIC)
 				accur_correct (real_chr, tutor.touch_time[tutor.ttidx-1]);
 		}
-
 		cursor_paint_char ("char_correct");
 	}
 	else
@@ -780,7 +803,6 @@ tutor_eval_forward (gunichar user_chr)
 		tutor.touch_time[tutor.ttidx] = g_timer_elapsed (tutor.tmr, NULL);
 		if (tutor.type != TT_BASIC)
 			accur_wrong (real_chr);
-
 		cursor_paint_char ("char_wrong");
 		tutor.n_errors += (int) n_touchs;
 		tutor_beep ();
@@ -970,43 +992,41 @@ tutor_calc_stats ()
 
 	accuracy = 100 * (1.0 - (gfloat) tutor.n_errors / tutor.n_touchs);
 	touchs_per_second = (gdouble) (tutor.n_touchs - tutor.n_errors) / tutor.elapsed_time;
-	velocity = 12 * touchs_per_second; // touched: new_WPM = 1.2 old_WPM
+	velocity = 12 * touchs_per_second;
 
-	if (tutor.type == TT_FLUID)
+	/* Average for touch timing
+	 */
+	sum = 0;
+	for (i = 2; i < tutor.ttidx; i++)
 	{
-		/*
-		 * "Magic" fluidness calculation
-		 */
-		sum = 0;
-		for (i = 2; i < tutor.ttidx; i++)
-		{
-			if (tutor.touch_time[i] <= 0)
-				tutor.touch_time[i] = 1.0e-8;
-			sample = sqrt (1 / tutor.touch_time[i]);
-			sum += sample;
-		}
-		if (i == 2)
-			i++;
-		average = sum / (i - 2);
-
-		sum = 0;
-		for (i = 2; i < tutor.ttidx; i++)
-		{
-			sample = sqrt (1 / tutor.touch_time[i]);
-			sum += (sample - average) * (sample - average);
-		}
-		if (i < 4)
-			i = 4;
-		standard_deviation = sqrt (sum / (i - 3));
-
-		if (average <= 0)
-			average = 1.0e-9;
-		fluidness = 100 * (1 - standard_deviation / average);
-		if (fluidness < 2)
-			fluidness = 2;
+		if (tutor.touch_time[i] <= 0)
+			tutor.touch_time[i] = 1.0e-8;
+		sample = sqrt (1 / tutor.touch_time[i]);
+		sum += sample;
 	}
-	else
-		fluidness = 0;
+	if (i == 2)
+		i++;
+	average = sum / (i - 2);
+	if (average <= 0)
+		average = 1.0e-9;
+
+	/* Standard deviation for touch timing
+	 */
+	sum = 0;
+	for (i = 2; i < tutor.ttidx; i++)
+	{
+		sample = sqrt (1 / tutor.touch_time[i]);
+		sum += (sample - average) * (sample - average);
+	}
+	if (i < 4)
+		i = 4;
+	standard_deviation = sqrt (sum / (i - 3));
+
+	/* "Magic" fluidness calculation
+	 */
+	fluidness = 100 * (1 - standard_deviation / average);
+	if (fluidness < 2)
+		fluidness = 2;
 	stat.score = 0;
 
 	/* Verify if logging is allowed 
@@ -1015,11 +1035,8 @@ tutor_calc_stats ()
 	if (tutor.type == TT_FLUID)
 		if (tutor.n_touchs < MIN_CHARS_TO_LOG)
 		{
-			gdk_beep ();
-			gdk_beep ();
-			contest_ps =
-				g_strdup_printf (_
-						 ("ps.: logging not performed for this session: "
+			gdk_display_beep (gdk_display_get_default ());
+			contest_ps = g_strdup_printf (_("ps.: logging not performed for this session: "
 						  "the number of typed characters (%i) must be greater than %i."),
 						 tutor.n_touchs, MIN_CHARS_TO_LOG);
 			may_log = FALSE;
@@ -1033,8 +1050,7 @@ tutor_calc_stats ()
 		if (tmp_locale != NULL)
 			setlocale (LC_NUMERIC, "C");
 
-		/*
-		 * Logging
+		/* Logging
 		 */
 		tmp_name =
 			g_strconcat (main_path_stats (), G_DIR_SEPARATOR_S "stat_", tutor_get_type_name (), ".txt",
@@ -1109,28 +1125,27 @@ tutor_calc_stats ()
 			g_message ("not able to log on this file:\n %s", tmp_name);
 		g_free (tmp_name);
 
+		/* Log the touch times deviation results of the last session
+		 */
+		tmp_name = g_strconcat (main_path_stats (), G_DIR_SEPARATOR_S, "deviation_", 
+				tutor_get_type_name (), ".txt", NULL);
+		if ((fh = (FILE *) g_fopen (tmp_name, "w")))
+		{
+			g_message ("writing touch timing deviation results at:\n %s", tmp_name);
+			fprintf (fh, "i\tdt(i)\t1/sqrt(dt(i))\tAver: %g\tStd: %g\tFluidity: %g\n", average, standard_deviation, fluidness);
+			for (i = 1; i < tutor.ttidx; i++)
+				fprintf (fh, "%i\t%g\t%g\n", i, tutor.touch_time[i], 
+						1.0 / sqrt(tutor.touch_time[i] > 0 ? tutor.touch_time[i] : 1.0e-9));
+			fclose (fh);
+		}
+		else
+			g_message ("not able to log on this file:\n %s", tmp_name);
+		g_free (tmp_name);
+
+		/* Fluidity specific results
+		 */
 		if (tutor.type == TT_FLUID)
 		{
-			/* Log the fluidness results of the last session
-			 */
-			tmp_name = g_build_filename (main_path_stats (), "deviation_fluid.txt", NULL);
-			if ((fh = (FILE *) g_fopen (tmp_name, "w")))
-			{
-				g_message ("writing further fluidness results at:\n %s", tmp_name);
-				fprintf (fh,
-					 "(i)\tdt(i)\tsqrt(1/dt(i))\tAverage:\t%g\tStd. dev.:\t%g\n",
-					 average, standard_deviation);
-				for (i = 1; i < tutor.ttidx; i++)
-					fprintf (fh, "%i\t%g\t%g\n", i, tutor.touch_time[i],
-						 sqrt (1 /
-						       (tutor.touch_time[i] >
-							0 ? tutor.touch_time[i] : 1.0e-9)));
-				fclose (fh);
-			}
-			else
-				g_message ("not able to log on this file:\n %s", tmp_name);
-			g_free (tmp_name);
-
 			/* Add results to Top 10
 			 */
 			tmp_name = main_preferences_get_string ("interface", "language");
@@ -1157,20 +1172,22 @@ tutor_calc_stats ()
 			top10_read_stats (LOCAL, -1);
 			if (tutor_char_distribution_approved ())
 			{
-				if (top10_compare_insert_stat (&stat, LOCAL))
+				if (top10_validate_stat (&stat))
 				{
-					contest_ps =
-						g_strdup (_
-							  ("ps.: you have entered the Top 10 list, great!"));
-					top10_write_stats (LOCAL, -1);
-					//if (main_preferences_get_boolean ("game", "autopublish") && UNIX_OK)
-					if (main_preferences_get_boolean ("game", "autopublish"))
+					if (top10_compare_insert_stat (&stat, LOCAL))
 					{
-						top10_show_stats (LOCAL);
-						top10_show_stats (GLOBAL);
-						top10_global_publish (NULL);
+						contest_ps = g_strdup (_("ps.: you have entered the Top 10 list, great!"));
+						top10_write_stats (LOCAL, -1);
+						if (main_preferences_get_boolean ("game", "autopublish"))
+						{
+							top10_show_stats (LOCAL);
+							top10_show_stats (GLOBAL);
+							top10_global_publish (NULL);
+						}
 					}
 				}
+				else
+					contest_ps = g_strdup (":-P");
 			}
 			else
 				contest_ps = g_strdup (_("ps.: the text you just typed doesn't seem to be similar"
@@ -1179,13 +1196,12 @@ tutor_calc_stats ()
 
 			/* Anyway, log also the scoring
 			 */
-			tmp_name =
-				g_build_filename (main_path_stats (), "scores_fluid.txt", NULL);
+			tmp_name = g_build_filename (main_path_stats (), "scores_fluid.txt", NULL);
 			assert_user_dir ();
 			if (!g_file_test (tmp_name, G_FILE_TEST_IS_REGULAR))
 			{
 				fh = (FILE *) g_fopen (tmp_name, "w");
-				fprintf (fh, "Score\tDate\tTime\tNumber of chars\tLanguage\n");
+				fprintf (fh, "Score\tDate\tTime\t# chars\tLang\n");
 			}
 			else
 				fh = (FILE *) g_fopen (tmp_name, "a");
@@ -1193,7 +1209,7 @@ tutor_calc_stats ()
 			{
 				ltime = localtime (&stat.when);
 				fprintf (fh,
-					 "%3.4f\t%i-%2.2i-%2.2i\t%2.2i:%2.2i\t%i\t%s\n",
+					 "%3.3f\t%i-%2.2i-%2.2i\t%2.2i:%2.2i\t%i\t%s\n",
 					 stat.score, (ltime->tm_year) + 1900, (ltime->tm_mon) + 1,
 					 (ltime->tm_mday), (ltime->tm_hour), (ltime->tm_min),
 					 stat.nchars, trans_get_current_language ());
@@ -1204,8 +1220,7 @@ tutor_calc_stats ()
 			g_free (tmp_name);
 		}
 
-		/*
-		 * Coming back to the right locale
+		/* Coming back to the right locale
 		 */
 		if (tmp_locale != NULL)
 		{
@@ -1214,14 +1229,12 @@ tutor_calc_stats ()
 		}
 	}
 
-	/*
-	 * Print statistics
+	/* Print statistics
 	 */
 	wg = get_wg ("text_tutor");
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
 
 	/* Begin the accuracy */
-	;
 	tmp_str = g_strconcat ("\n", _("STATISTICS"), "\n",
 			       _("Elapsed time:"), " %i ",
 			       dngettext (PACKAGE, "minute and", "minutes and", minutes),
@@ -1345,27 +1358,28 @@ tutor_calc_stats ()
 		break;
 	case TT_FLUID:
 		fluid_comment (accuracy, velocity, fluidness);
-
 		if (contest_ps != NULL)
 		{
-			//if (UNIX_OK)
-			{
-				gtk_text_buffer_insert_at_cursor (buf, "\n", 1);
-				gtk_text_buffer_insert_at_cursor (buf, contest_ps, strlen (contest_ps));
-			}
+			gtk_text_buffer_insert_at_cursor (buf, "\n", 1);
+			gtk_text_buffer_insert_at_cursor (buf, contest_ps, strlen (contest_ps));
 			g_free (contest_ps);
 		}
 		break;
 	}
 	g_free (tmp_str2);
 
+	/* Is this really needed? We don't remember... FIXME (remove?)
+	gtk_text_buffer_get_bounds (buf, &start, &end);
+	gtk_text_buffer_apply_tag_by_name (buf, "char_keep_wrap", &start, &end);
+	*/
+
 	gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (wg), gtk_text_buffer_get_insert (buf));
 }
 
 /**********************************************************************
  * Ensure the user is not trying to type with weird texts in the fluidness contest
  */
-#define DECEIVENESS_LIMIT 0.195 // 0.135
+#define DECEIVENESS_LIMIT 0.205
 gboolean
 tutor_char_distribution_approved ()
 {
@@ -1546,11 +1560,10 @@ tutor_draw_paragraph (gchar * utf8_text)
 	static gchar *tmp1 = NULL;
 	static gchar *tmp2 = NULL;
 	gchar *ptr;
-	GtkWidget *wg;
 	GtkTextBuffer *buf;
 
-	g_free (tmp1);
-	g_free (tmp2);
+	//g_free (tmp1);
+	//g_free (tmp2);
 
 	if (g_utf8_strrchr (utf8_text, -1, L'\n') == NULL)
 	{
@@ -1566,12 +1579,12 @@ tutor_draw_paragraph (gchar * utf8_text)
 	else
 		g_error ("draw_paragraph () ==> string error");
 
-	wg = get_wg ("text_tutor");
-	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wg));
-
 	tmp2 = g_strconcat (tmp1, keyb_get_utf8_paragraph_symbol (), "\n", NULL);
+	g_free (tmp1);
 
+	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (get_wg ("text_tutor")));
 	gtk_text_buffer_insert_at_cursor (buf, tmp2, -1);
+	g_free (tmp2);
 }
 
 /**********************************************************************
@@ -1655,7 +1668,7 @@ tutor_other_rename (const gchar *new_tx, const gchar *old_tx)
 		if (g_file_test (new_file, G_FILE_TEST_IS_REGULAR))
 		{
 			g_message ("File already exists, not renaming.\n\t%s\n", new_file);
-			gdk_beep ();
+			gdk_display_beep (gdk_display_get_default ());
 		}
 		else
 		{
@@ -1703,11 +1716,8 @@ tutor_message (gchar * mesg)
 void
 tutor_beep ()
 {
-	GtkWidget *wg;
-
-	wg = get_wg ("togglebutton_tutor_beep");
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wg)))
-		gdk_beep ();
+	if (main_preferences_get_boolean ("tutor", "tutor_beep"))
+		gdk_display_beep (gdk_display_get_default ());
 }
 
 /**********************************************************************
