@@ -1,14 +1,15 @@
-/*****************************************************************************/
-/*  Klavaro - a flexible touch typing tutor                                  */
-/*  Copyright (C) 2005, 2006, 2007, 2008 Felipe Castro                       */
-/*  Copyright (C) 2009, 2010, 2011, 2012, 2013 The Free Software Foundation  */
-/*                                                                           */
-/*  This program is free software, licensed under the terms of the GNU       */
-/*  General Public License as published by the Free Software Foundation,     */
-/*  either version 3 of the License, or (at your option) any later version.  */
-/*  You should have received a copy of the GNU General Public License        */
-/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
-/*****************************************************************************/
+/**************************************************************************/
+/*  Klavaro - a flexible touch typing tutor                               */
+/*  Copyright (C) from 2005 until 2008 Felipe Castro                      */
+/*  Copyright (C) from 2009 until 2019 The Free Software Foundation       */
+/*                                                                        */
+/*  This program is free software, licensed under the terms of the GNU    */
+/*  General Public License as published by the Free Software Foundation,  */
+/*  either version 3 of the License, or (at your option) any later        */
+/*  version. You should have received a copy of the GNU General Public    */
+/*  License along with this program. If not,                              */
+/*  see <https://www.gnu.org/licenses/>.                                  */
+/**************************************************************************/
 
 /*
  * Velocity exercise
@@ -314,7 +315,7 @@ velo_text_write_to_file (gchar * text_raw, gboolean overwrite)
 	assert_user_dir ();
 	if (!(fh_destiny = (FILE *) g_fopen (dict_path, "w")))
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default ());
 		g_warning ("couldn't create the file:\n <%s>", dict_path);
 		if (overwrite == FALSE)
 		{
@@ -358,14 +359,14 @@ velo_create_dict (gchar * file_name, gboolean overwrite)
 
 	if (!file_name)
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default ());
 		g_warning ("velo_create_dict(): null file name as first argument.");
 		return;
 	}
 
 	if (!g_file_get_contents (file_name, &text_raw, NULL, NULL))
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default ());
 		g_warning ("couldn't read the file:\n <%s>", file_name);
 		return;
 	}
@@ -375,7 +376,7 @@ velo_create_dict (gchar * file_name, gboolean overwrite)
 	assert_user_dir ();
 	if (!(fh_destiny = (FILE *) g_fopen (dict_path, "w")))
 	{
-		gdk_beep ();
+		gdk_display_beep (gdk_display_get_default ());
 		g_warning ("couldn't create the file:\n <%s>", dict_path);
 		if (overwrite == FALSE)
 		{
@@ -450,7 +451,7 @@ velo_comment (gdouble accuracy, gdouble velocity)
 		/* Translators: feel free to change the reference to that martial art, if you find another good analogy. */
 		tmp_str = g_strdup_printf (_(" Kung-fu mastery!\n Can you fly at %.0f WPM?\n"), tutor_goal_level(7));
 	else
-		/* Translators: Dvorak here means that infamous ergonomic keyboard layout. */
+		/* Translators: Dvorak here means that infamous ergonomic keyboard layout. Feel free to change the speed mastery analogy. */
 		tmp_str = g_strdup (_(" Dvorak master!\n"
 				   " I have no words to express my admiration!\n"));
 
